UAWOW = LibStub("AceAddon-3.0"):NewAddon("UAWOW", "AceConsole-3.0")

UAWOW.FONT = "Interface\\AddOns\\UAWOW\\font\\ukrtimes.ttf";
UAWOW.DEBUG = false -- false when release

UAWOW.property = {}

function UAWOW:OnInitialize()
    self.version = GetAddOnMetadata("UAWOW", "Version")
    print(string.format("Welcome to the world of Ukrainian World of Warcraft  Ver: %s.\nSetting is /cua or /UAWOW.", self.version))

    self.db = LibStub('AceDB-3.0'):New("UAWOWDB")
	self.db:RegisterDefaults({
        profile = {
            quest       = { questlog = true, worldmap = true, gossip = true, questlog_movable = false },
            item        = { tooltip = true },
            spell       = { tooltip = true, buff = true },
            achievement = { tooltip = true, advice = true },
            developer   = self.DEBUG, -- false when release
            development = { debugger = self.DEBUG }, -- false when release
			config      = {fontsize = 0, tooltip = true},
        }
    })

    -- Quest --
    UAWOWQuestLog:OnInitialize()
    UAWOWWorldMapQuest:OnInitialize()
    UAWOWQuestGossip:OnInitialize()

    -- ToolTip --
    UAWOWGameToolTip:OnInitialize()
    UAWOWItemRefTooltip:OnInitialize()

    UAWOWBuffToolTip:OnInitialize()
    UAWOWGlyphToolTip:OnInitialize()
    UAWOWGarrisonToolTip:OnInitialize()

end

function UAWOW:OnEnable()
    self:DebugLog("UAWOW: OnEnable.");

    -- Option --
    self:SetupOptions()

    -- Quest --
    UAWOWQuestLog:OnEnable()
    UAWOWWorldMapQuest:OnEnable()
    UAWOWQuestGossip:OnEnable()

    -- ToolTip --
    UAWOWGameToolTip:OnEnable()
    UAWOWItemRefTooltip:OnEnable()

	-- Buff	
    UAWOWBuffToolTip:OnEnable()
	-- Glyph
    UAWOWGlyphToolTip:OnEnable()
	
	UAWOWGarrisonToolTip:OnEnable()

end

function UAWOW:OnDisable()
    self:DebugLog("UAWOW: OnDisable.");

    -- Quest --
    UAWOWQuestLog:OnDisable()
    UAWOWWorldMapQuest:OnDisable()
    UAWOWQuestGossip:OnDisable()

    -- ToolTip --
    UAWOWGameToolTip:OnDisable()
    UAWOWItemRefTooltip:OnDisable()

	-- Buff	
    UAWOWBuffToolTip:OnDisable()

	-- Glyph
    UAWOWGlyphToolTip:OnDisable()
	
	UAWOWGarrisonToolTip:OnDisable()

end

function UAWOW:LoadAddOn(addon)
    return true
end

function UAWOW:FontSize()
	return self.db.profile.config.fontsize
end

function UAWOW:isShowQuest()
	return self.db.profile.quest.questlog
end

function UAWOW:isShowTooltip()
	return self.db.profile.config.tooltip
end

function UAWOW:uc(str)
    return string.gsub(str, "(%w)", function(s)
        return string.upper(s)
    end, 1)
end

function UAWOW:lc(str)
    return string.gsub(str, "(%w)", function(s)
        return string.lower(s)
    end, 1)
end

function UAWOW:SetupOptions()
    function GetOptions()
        return {
            type = "group",
            name = "Ukrainian World of Warcraft",
			order = 1,
            args = {         
				Header1 = {
					type = "header",
					name = "Plugin Options - Please /reload to apply change.",
					order = 2,
				},
				ShowQuestLog = {
					type = "toggle",
					name = "Show Quest Log",
					order = 10,
					set = function(info, value)
						self.db.profile.quest.questlog = value;
					end,
					get = function(info)
						return self.db.profile.quest.questlog;
					end,					
				},
				Spacer2 = {
					type = "description",
					name = "Show Ukrainian Quest Log",
					fontSize = "medium",
					order = 15,
				},
				ShowToolTip = {
					type = "toggle",
					name = "Show Tool Tip",
					order = 20,
					set = function(info, value)
						self.db.profile.config.tooltip = value;
						self.db.profile.item.tooltip = value;
						self.db.profile.spell.tooltip = value;
						self.db.profile.achievement.tooltip = value;
					end,
					get = function(info)
						return self.db.profile.config.tooltip;
					end,					
				},
				Spacer3 = {
					type = "description",
					name = "Show Ukrainian Tool Tip for Item/Spell/Achievement",
					fontSize = "medium",
					order = 25,
				},
				FontSizeSlider = {
					type = "range",
					name = "Font Size",
					min = 0,
					max = 10,
					step = 1,
					order = 40,
					set = function(info, value)
						self.db.profile.config.fontsize = value;
					end,
					get = function(info)
						return self.db.profile.config.fontsize;
					end,					
				},
				Spacer_FontSize = {
					type = "description",
					name = "Make Font Size Bigger in Quest Log and Tool Tip. (0=Default Size)\nPlease /reload to apply change.",
					fontSize = "medium",
					order = 45,
				},
            },
        }
    end

    LibStub("AceConfig-3.0"):RegisterOptionsTable("UAWOW", GetOptions())	
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("UAWOW");

    self:RegisterChatCommand("cua", "ChatCommand")
    self:RegisterChatCommand("UAWOW", "ChatCommand")
end

function UAWOW:ChatCommand(input)
    if input == 'debug' then
        -- Debug Window --
        if self.db.profile.development.debugger then
            UAWOWDebugFrame:Show()
        end
    else
        self:OpenConfigDialog()
    end
end

function UAWOW:OpenConfigDialog()
    LibStub("AceConfigDialog-3.0"):Open("UAWOW")
end

function UAWOW:DebugLog(args)
    if not self.DEBUG then
        return
    end

    if type(args) == 'string' then
        self:Print(args)
        return
    end

    if type(args) == 'number' then
        self:Print(tostring(args))
        return
    end

    if type(args) == 'table' then
        self:Print("----------")
        for n, v in pairs(args) do
            self:Print(n .. ' : ' .. v)
        end
        self:Print("----------")
        return
    end
end
